<?php declare(strict_types=1);

namespace unit\php\Core\Content\Mail\Service;

use Doctrine\DBAL\Connection;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Shopware\Core\Checkout\Document\DocumentCollection;
use Shopware\Core\Checkout\Document\DocumentEntity;
use Shopware\Core\Checkout\Document\DocumentService;
use Shopware\Core\Checkout\Document\GeneratedDocument;
use Shopware\Core\Checkout\Document\Renderer\RenderedDocument;
use Shopware\Core\Checkout\Document\Service\DocumentGenerator;
use Shopware\Core\Content\Mail\Service\MailAttachmentsBuilder;
use Shopware\Core\Content\MailTemplate\Aggregate\MailTemplateMedia\MailTemplateMediaCollection;
use Shopware\Core\Content\MailTemplate\Aggregate\MailTemplateMedia\MailTemplateMediaEntity;
use Shopware\Core\Content\MailTemplate\MailTemplateEntity;
use Shopware\Core\Content\MailTemplate\Subscriber\MailSendSubscriberConfig;
use Shopware\Core\Content\Media\MediaCollection;
use Shopware\Core\Content\Media\MediaEntity;
use Shopware\Core\Content\Media\MediaService;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\EntitySearchResult;
use Shopware\Core\Framework\Feature;
use Shopware\Core\Framework\Uuid\Uuid;

/**
 * @internal
 * @covers \Shopware\Core\Content\Mail\Service\MailAttachmentsBuilder
 */
class MailAttachmentsBuilderTest extends TestCase
{
    /**
     * @var MockObject|MediaService
     */
    private $mediaService;

    /**
     * @var MockObject|EntityRepositoryInterface
     */
    private $mediaRepository;

    /**
     * @var MockObject|EntityRepositoryInterface
     */
    private $documentRepository;

    /**
     * @var MockObject|DocumentGenerator
     */
    private $documentGenerator;

    /**
     * @var MockObject|DocumentService
     */
    private $documentService;

    /**
     * @var MockObject|Connection
     */
    private $connection;

    private MailAttachmentsBuilder $attachmentsBuilder;

    public function setUp(): void
    {
        $this->mediaService = $this->createMock(MediaService::class);
        $this->mediaRepository = $this->createMock(EntityRepositoryInterface::class);
        $this->documentRepository = $this->createMock(EntityRepositoryInterface::class);
        $this->documentGenerator = $this->createMock(DocumentGenerator::class);
        $this->documentService = $this->createMock(DocumentService::class);
        $this->connection = $this->createMock(Connection::class);

        $this->attachmentsBuilder = new MailAttachmentsBuilder(
            $this->mediaService,
            $this->mediaRepository,
            $this->documentRepository,
            $this->documentGenerator,
            $this->documentService,
            $this->connection
        );
    }

    public function testBuildTemplateMediaAttachments(): void
    {
        $context = Context::createDefaultContext();
        $mailTemplate = new MailTemplateEntity();
        $extension = new MailSendSubscriberConfig(false);

        $mediaA = new MailTemplateMediaEntity();
        $mediaA->setId(Uuid::randomHex());
        $mediaA->setMedia(new MediaEntity());
        $mediaA->setLanguageId($context->getLanguageId());
        $mediaB = new MailTemplateMediaEntity();
        $mediaB->setId(Uuid::randomHex());
        $mediaC = new MailTemplateMediaEntity();
        $mediaC->setId(Uuid::randomHex());
        $mediaC->setMedia(new MediaEntity());
        $mediaC->setLanguageId($context->getLanguageId());

        $mailTemplate->setMedia(new MailTemplateMediaCollection([$mediaA, $mediaB, $mediaC]));

        $this->mediaService
            ->expects(static::exactly(2))
            ->method('getAttachment')
            ->withConsecutive([$mediaA->getMedia(), $context], [$mediaC->getMedia(), $context])
            ->willReturnOnConsecutiveCalls(
                [
                    'content' => 'foo',
                    'fileName' => 'foo',
                    'mimeType' => 'foo',
                ],
                [
                    'content' => 'bar',
                    'fileName' => 'bar',
                    'mimeType' => 'bar',
                ]
            );

        $attachments = $this->attachmentsBuilder->buildAttachments($context, $mailTemplate, $extension, [], Uuid::randomHex());

        static::assertEquals(
            [
                [
                    'content' => 'foo',
                    'fileName' => 'foo',
                    'mimeType' => 'foo',
                ],
                [
                    'content' => 'bar',
                    'fileName' => 'bar',
                    'mimeType' => 'bar',
                ],
            ],
            $attachments
        );
    }

    public function testBuildTemplateDocumentAttachments(): void
    {
        $context = Context::createDefaultContext();
        $mailTemplate = new MailTemplateEntity();
        $idA = Uuid::randomHex();
        $idB = Uuid::randomHex();
        $idC = Uuid::randomHex();
        $idD = Uuid::randomHex();
        $idE = Uuid::randomHex();
        $idF = Uuid::randomHex();
        $extension = new MailSendSubscriberConfig(false, [$idA, $idB], [$idE, $idF]);
        $eventConfig = ['documentTypeIds' => [$idA, $idB]];
        $orderId = Uuid::randomHex();

        $this->connection
            ->expects(static::once())
            ->method('fetchAllAssociative')
            ->with(
                static::anything(),
                ['orderId' => Uuid::fromHexToBytes($orderId), 'documentTypeIds' => Uuid::fromHexToBytesList($eventConfig['documentTypeIds'])],
                ['documentTypeIds' => Connection::PARAM_STR_ARRAY]
            )
            ->willReturn([
                ['doc_type' => 'foo', 'doc_id' => '1'],
                ['doc_type' => 'bar', 'doc_id' => '2'],
                ['doc_type' => 'foo', 'doc_id' => '3'],
                ['doc_type' => 'foo', 'doc_id' => $idC],
                ['doc_type' => 'bar', 'doc_id' => $idD],
            ]);

        if (Feature::isActive('v6.5.0.0')) {
            $document = new RenderedDocument();
            $document->setContent('');
            $this->documentGenerator
                ->expects(static::exactly(4))
                ->method('readDocument')
                ->withConsecutive([$idA, $context], [$idB, $context], [$idC, $context], [$idD, $context])
                ->willReturn($document);
        } else {
            $criteria = new Criteria([$idA, $idB, $idC, $idD]);
            $criteria->setTitle('send-mail::load-attachments');
            $criteria->addAssociation('documentMediaFile');
            $criteria->addAssociation('documentType');

            $entities = array_map(static function (string $id): DocumentEntity {
                $document = new DocumentEntity();
                $document->setId($id);

                return $document;
            }, [$idA, $idB, $idC, $idD]);

            $this->documentRepository
                ->expects(static::once())
                ->method('search')
                ->with($criteria, $context)
                ->willReturn(new EntitySearchResult('document', 4, new DocumentCollection($entities), null, $criteria, $context));

            $document = new GeneratedDocument();
            $document->setFileBlob('');
            $document->setFilename('');
            $document->setContentType('application/pdf');
            $this->documentService
                ->expects(static::exactly(4))
                ->method('getDocument')
                ->withConsecutive([$entities[0], $context], [$entities[1], $context], [$entities[2], $context], [$entities[3], $context])
                ->willReturn($document);
        }

        $criteria = new Criteria($extension->getMediaIds());
        $criteria->setTitle('send-mail::load-media');
        $entities = array_map(static function (string $id): MediaEntity {
            $media = new MediaEntity();
            $media->setId($id);

            return $media;
        }, $extension->getMediaIds());

        $this->mediaRepository
            ->expects(static::once())
            ->method('search')
            ->with($criteria, $context)
            ->willReturn(new EntitySearchResult('media', 2, new MediaCollection($entities), null, $criteria, $context));

        $this->mediaService
            ->expects(static::exactly(2))
            ->method('getAttachment')
            ->withConsecutive([$entities[0], $context], [$entities[1], $context])
            ->willReturnOnConsecutiveCalls(
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idE,
                ],
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idF,
                ]
            );

        $attachments = $this->attachmentsBuilder->buildAttachments($context, $mailTemplate, $extension, $eventConfig, $orderId);

        static::assertEquals(
            [
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idA,
                ],
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idB,
                ],
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idC,
                ],
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idD,
                ],
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idE,
                ],
                [
                    'content' => '',
                    'fileName' => '',
                    'mimeType' => 'application/pdf',
                    'id' => $idF,
                ],
            ],
            $attachments
        );
    }
}
