<?php declare(strict_types=1);

namespace Shopware\Tests\Unit\Core\System\Snippet;

use Doctrine\DBAL\Connection;
use PHPUnit\Framework\TestCase;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Uuid\Uuid;
use Shopware\Core\System\Snippet\Files\SnippetFileCollection;
use Shopware\Core\System\Snippet\Filter\SnippetFilterFactory;
use Shopware\Core\System\Snippet\SnippetService;
use Shopware\Storefront\Theme\SalesChannelThemeLoader;
use Shopware\Storefront\Theme\StorefrontPluginConfiguration\StorefrontPluginConfiguration;
use Shopware\Storefront\Theme\StorefrontPluginConfiguration\StorefrontPluginConfigurationCollection;
use Shopware\Storefront\Theme\StorefrontPluginRegistry;
use Shopware\Tests\Unit\Core\System\Snippet\Mock\MockSnippetFile;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Translation\MessageCatalogue;

/**
 * @internal
 *
 * @covers \Shopware\Core\System\Snippet\SnippetService
 */
class SnippetServiceTest extends TestCase
{
    private SnippetFileCollection $snippetCollection;

    /**
     * @var Connection&\PHPUnit\Framework\MockObject\MockObject
     */
    private $connection;

    protected function setUp(): void
    {
        $this->connection = $this->createMock(Connection::class);
        $this->snippetCollection = new SnippetFileCollection();
        $this->addThemes();
    }

    /**
     * @dataProvider getStorefrontSnippetsDataProvider
     *
     * @param list<string> $catalogMessages
     * @param string|false $fetchLocaleResult
     * @param \Throwable|list<string> $expected
     * @param list<string> $databaseSnippets
     */
    public function testGetStorefrontSnippets(
        $expected = [],
        $fetchLocaleResult = 'en-GB',
        array $catalogMessages = [],
        ?string $fallbackLocale = null,
        ?string $salesChannelId = null,
        ?bool $withThemeRegistry = true,
        ?string $usedTheme = null,
        ?array $databaseSnippets = []
    ): void {
        $classExists = class_exists(StorefrontPluginRegistry::class);

        if ($withThemeRegistry && !$classExists) {
            $this->testGetStorefrontSnippetsWithoutThemeRegistry();

            return;
        }

        if ($expected instanceof \Throwable) {
            static::expectException(\get_class($expected));
        }

        $container = $this->createMock(ContainerInterface::class);
        $container->method('has')->with(StorefrontPluginRegistry::class)->willReturn($withThemeRegistry);
        $themeLoader = null;
        $this->connection->expects(static::once())->method('fetchOne')->willReturn($fetchLocaleResult);

        if ($withThemeRegistry) {
            $plugins = new StorefrontPluginConfigurationCollection();

            foreach (['Storefront', 'SwagTheme'] as $technicalName) {
                $theme = new StorefrontPluginConfiguration($technicalName);
                $theme->setIsTheme(true);
                $plugins->add($theme);
            }

            $themeRegistry = $this->createMock(StorefrontPluginRegistry::class);
            $themeRegistry->expects(static::once())->method('getConfigurations')->willReturn($plugins);
            $container->expects(static::exactly(1))->method('get')->with(StorefrontPluginRegistry::class)->willReturn($themeRegistry);
        }

        if ($salesChannelId !== null) {
            $themeLoader = $this->createMock(SalesChannelThemeLoader::class);
            $themeLoader->expects(static::once())->method('load')->with($salesChannelId)->willReturn([
                'themeName' => $usedTheme ?? 'Storefront',
            ]);
        }

        if (!empty($databaseSnippets)) {
            $this->connection->expects(static::once())->method('fetchAllKeyValue')->willReturn($databaseSnippets);
        }

        $snippetService = new SnippetService(
            $this->connection,
            $this->snippetCollection,
            $this->createMock(EntityRepositoryInterface::class),
            $this->createMock(EntityRepositoryInterface::class),
            $this->createMock(EntityRepositoryInterface::class),
            $this->createMock(SnippetFilterFactory::class),
            $container,
            $themeLoader,
        );

        $catalog = new MessageCatalogue((string) $fetchLocaleResult, ['messages' => $catalogMessages]);

        $snippets = $snippetService->getStorefrontSnippets($catalog, Uuid::randomHex(), $fallbackLocale, $salesChannelId);

        static::assertEquals($expected, $snippets);
    }

    public function testGetStorefrontSnippetsWithoutThemeRegistry(): void
    {
        $locale = 'de-DE';
        $snippetSetId = Uuid::randomHex();
        $catalog = new MessageCatalogue($locale, []);

        $container = $this->createMock(ContainerInterface::class);
        $container->expects(static::once())->method('has')->with(StorefrontPluginRegistry::class)->willReturn(false);

        $snippetService = new SnippetService(
            $this->connection,
            $this->snippetCollection,
            $this->createMock(EntityRepositoryInterface::class),
            $this->createMock(EntityRepositoryInterface::class),
            $this->createMock(EntityRepositoryInterface::class),
            $this->createMock(SnippetFilterFactory::class),
            $container
        );

        $snippets = $snippetService->getStorefrontSnippets($catalog, $snippetSetId, $locale);

        static::assertSame([
            'title' => 'SwagTheme DE',
        ], $snippets);
    }

    /**
     * @return iterable<string, array<mixed>>
     */
    public function getStorefrontSnippetsDataProvider(): iterable
    {
        yield 'with unknown snippet id' => [
            'expected' => new \InvalidArgumentException('No snippetSet with id "%s" found'),
            'fetchLocaleResult' => false,
            'catalogMessages' => [],
            'fallbackLocale' => null,
            'salesChannelId' => null,
            'withThemeRegistry' => false,
        ];

        yield 'with messages from catalog' => [
            'expected' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Storefront EN',
            ],
            'fetchLocaleResult' => 'en-GB',
            'catalogMessages' => [
                'catalog_key' => 'Catalog DE',
            ],
        ];

        yield 'fallback snippets are used if no localized snippet found' => [
            'expected' => [
                'title' => 'Storefront DE',
            ],
            'fetchLocaleResult' => 'vi-VN',
            'catalogMessages' => [],
            'fallbackLocale' => 'de-DE',
        ];

        yield 'fallback snippets are overrided by catalog messages' => [
            'expected' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Catalog title',
            ],
            'fetchLocaleResult' => 'vi-VN',
            'catalogMessages' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Catalog title',
            ],
            'fallbackLocale' => 'en-GB',
        ];

        yield 'fallback snippets, catalog messages are overrided by localized snippets' => [
            'expected' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Storefront DE',
            ],
            'fetchLocaleResult' => 'de-DE',
            'catalogMessages' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Catalog title',
            ],
            'fallbackLocale' => 'en-GB',
        ];

        yield 'fallback snippets, catalog message, localized snippets are overrided by database snippets' => [
            'expected' => [
                'title' => 'Database title',
                'catalog_key' => 'Catalog DE',
            ],
            'fetchLocaleResult' => 'de-DE',
            'catalogMessages' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Catalog title',
            ],
            'fallbackLocale' => 'en-GB',
            'salesChannelId' => null,
            'withThemeRegistry' => true,
            'usedTheme' => null,
            'databaseSnippets' => [
                'title' => 'Database title',
            ],
        ];

        yield 'with sales channel id without theme' => [
            'expected' => [
                'title' => 'Storefront DE',
            ],
            'fetchLocaleResult' => 'de-DE',
            'catalogMessages' => [],
            'fallbackLocale' => 'en-GB',
            'salesChannelId' => Uuid::randomHex(),
            'withThemeRegistry' => true,
            'usedTheme' => null,
            'databaseSnippets' => [],
        ];

        yield 'with sales channel id and theme' => [
            'expected' => [
                'title' => 'SwagTheme DE',
            ],
            'fetchLocaleResult' => 'de-DE',
            'catalogMessages' => [],
            'fallbackLocale' => 'en-GB',
            'salesChannelId' => Uuid::randomHex(),
            'withThemeRegistry' => true,
            'usedTheme' => 'SwagTheme',
        ];

        yield 'theme snippets are overrided by database snippets' => [
            'expected' => [
                'title' => 'Database title',
                'catalog_key' => 'Catalog DE',
            ],
            'fetchLocaleResult' => 'de-DE',
            'catalogMessages' => [
                'catalog_key' => 'Catalog DE',
                'title' => 'Catalog title',
            ],
            'fallbackLocale' => 'en-GB',
            'salesChannelId' => Uuid::randomHex(),
            'withThemeRegistry' => true,
            'usedTheme' => 'SwagTheme',
            'databaseSnippets' => [
                'title' => 'Database title',
            ],
        ];
    }

    private function addThemes(): void
    {
        $this->snippetCollection->add(new MockSnippetFile('storefront.de-DE', 'de-DE', '{}', true, 'Storefront'));
        $this->snippetCollection->add(new MockSnippetFile('storefront.en-GB', 'en-GB', '{}', true, 'Storefront'));
        $this->snippetCollection->add(new MockSnippetFile('swagtheme.de-DE', 'de-DE', '{}', true, 'SwagTheme'));
        $this->snippetCollection->add(new MockSnippetFile('swagtheme.en-GB', 'en-GB', '{}', true, 'SwagTheme'));
    }
}
